#ifndef SQLFORMATTERSIMPLE_GLOBAL_H
#define SQLFORMATTERSIMPLE_GLOBAL_H

#include <QtCore/qglobal.h>

#if defined(SQLFORMATTERSIMPLE_LIBRARY)
#  define SQLFORMATTERSIMPLESHARED_EXPORT Q_DECL_EXPORT
#else
#  define SQLFORMATTERSIMPLESHARED_EXPORT Q_DECL_IMPORT
#endif

#endif // SQLFORMATTERSIMPLE_GLOBAL_H
