#ifndef REGEXPIMPORT_GLOBAL_H
#define REGEXPIMPORT_GLOBAL_H

#include <QtCore/qglobal.h>

#if defined(REGEXPIMPORT_LIBRARY)
#  define REGEXPIMPORTSHARED_EXPORT Q_DECL_EXPORT
#else
#  define REGEXPIMPORTSHARED_EXPORT Q_DECL_IMPORT
#endif

#endif // REGEXPIMPORT_GLOBAL_H
