#ifndef DBSQLITECIPHER_GLOBAL_H
#define DBSQLITECIPHER_GLOBAL_H

#include <QtCore/qglobal.h>

#if defined(DBSQLITECIPHER_LIBRARY)
#  define DBSQLITECIPHERSHARED_EXPORT Q_DECL_EXPORT
#else
#  define DBSQLITECIPHERSHARED_EXPORT Q_DECL_IMPORT
#endif

#endif // DBSQLITECIPHER_GLOBAL_H
