#ifndef CONFIGMIGRATION_GLOBAL_H
#define CONFIGMIGRATION_GLOBAL_H

#include <QtCore/qglobal.h>

#if defined(CONFIGMIGRATION_LIBRARY)
#  define CONFIGMIGRATIONSHARED_EXPORT Q_DECL_EXPORT
#else
#  define CONFIGMIGRATIONSHARED_EXPORT Q_DECL_IMPORT
#endif

#endif // CONFIGMIGRATION_GLOBAL_H
