#ifndef MULTIEDITORIMAGE_GLOBAL_H
#define MULTIEDITORIMAGE_GLOBAL_H

#include <QtCore/qglobal.h>

#if defined(MULTIEDITORIMAGE_LIBRARY)
#  define MULTIEDITORIMAGE_EXPORT Q_DECL_EXPORT
#else
#  define MULTIEDITORIMAGE_EXPORT Q_DECL_IMPORT
#endif

#endif // MULTIEDITORIMAGE_GLOBAL_H
