#ifndef SCRIPTINGTCL_GLOBAL_H
#define SCRIPTINGTCL_GLOBAL_H

#include <QtCore/qglobal.h>

#if defined(SCRIPTINGTCL_LIBRARY)
#  define SCRIPTINGTCLSHARED_EXPORT Q_DECL_EXPORT
#else
#  define SCRIPTINGTCLSHARED_EXPORT Q_DECL_IMPORT
#endif

#endif // SCRIPTINGTCL_GLOBAL_H
